package com.sun.xml.internal.ws.db;

import com.oracle.webservices.internal.api.message.MessageContext;
import java.io.InputStream;
import java.io.IOException;
import com.sun.xml.internal.ws.api.pipe.ContentType;
import java.io.OutputStream;
import com.sun.xml.internal.ws.binding.BindingImpl;
import com.sun.xml.internal.ws.api.databinding.EndpointCallBridge;
import com.sun.xml.internal.ws.wsdl.writer.WSDLGenerator;
import com.sun.xml.internal.ws.api.databinding.WSDLGenInfo;
import com.sun.xml.internal.ws.api.databinding.ClientCallBridge;
import com.sun.xml.internal.ws.api.message.Message;
import javax.xml.ws.WebServiceFeature;
import com.oracle.webservices.internal.api.databinding.JavaCallInfo;
import com.sun.xml.internal.ws.wsdl.DispatchException;
import com.sun.xml.internal.ws.api.model.WSDLOperationMapping;
import com.sun.xml.internal.ws.api.message.Packet;
import com.sun.xml.internal.ws.wsdl.ActionBasedOperationSignature;
import com.sun.xml.internal.ws.client.sei.StubAsyncHandler;
import com.sun.xml.internal.ws.api.model.MEP;
import com.sun.xml.internal.ws.api.model.SEIModel;
import java.util.Iterator;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLPort;
import com.sun.xml.internal.ws.model.RuntimeModeler;
import java.util.HashMap;
import com.sun.xml.internal.ws.api.databinding.DatabindingConfig;
import com.sun.xml.internal.ws.api.message.MessageContextFactory;
import com.sun.xml.internal.ws.api.pipe.Codec;
import com.sun.xml.internal.ws.wsdl.OperationDispatcher;
import com.sun.xml.internal.ws.server.sei.TieHandler;
import com.sun.xml.internal.ws.model.JavaMethodImpl;
import com.sun.xml.internal.ws.client.sei.StubHandler;
import java.lang.reflect.Method;
import java.util.Map;
import com.sun.xml.internal.ws.model.AbstractSEIModelImpl;
import com.sun.xml.internal.ws.api.databinding.Databinding;

public final class DatabindingImpl implements Databinding
{
    AbstractSEIModelImpl seiModel;
    Map<Method, StubHandler> stubHandlers;
    Map<JavaMethodImpl, TieHandler> wsdlOpMap;
    Map<Method, TieHandler> tieHandlers;
    OperationDispatcher operationDispatcher;
    OperationDispatcher operationDispatcherNoWsdl;
    boolean clientConfig;
    Codec codec;
    MessageContextFactory packetFactory;
    
    public DatabindingImpl(DatabindingProviderImpl p, DatabindingConfig config) {
        this.wsdlOpMap = new HashMap();
        this.tieHandlers = new HashMap();
        this.clientConfig = false;
        this.packetFactory = null;
        RuntimeModeler modeler = new RuntimeModeler(config);
        modeler.setClassLoader(config.getClassLoader());
        this.seiModel = modeler.buildRuntimeModel();
        WSDLPort wsdlport = config.getWsdlPort();
        this.packetFactory = new MessageContextFactory(this.seiModel.getWSBinding().getFeatures());
        this.clientConfig = this.isClientConfig(config);
        if (this.clientConfig) {
            this.initStubHandlers();
        }
        this.seiModel.setDatabinding(this);
        if (wsdlport != null) {
            this.freeze(wsdlport);
        }
        if (this.operationDispatcher == null) {
            this.operationDispatcherNoWsdl = new OperationDispatcher(null, this.seiModel.getWSBinding(), this.seiModel);
        }
        for (JavaMethodImpl jm : this.seiModel.getJavaMethods()) {
            if (!jm.isAsync()) {
                TieHandler th = new TieHandler(jm, this.seiModel.getWSBinding(), this.packetFactory);
                this.wsdlOpMap.put(jm, th);
                this.tieHandlers.put(th.getMethod(), th);
            }
            else {
                continue;
            }
        }
    }
    
    private boolean isClientConfig(DatabindingConfig config) {
        return config.getContractClass() != null && config.getContractClass().isInterface() && (config.getEndpointClass() == null || config.getEndpointClass().isInterface());
    }
    
    public void freeze(WSDLPort port) {
        if (this.clientConfig) {
            return;
        }
        synchronized (this) {
            if (this.operationDispatcher == null) {
                this.operationDispatcher = ((port == null) ? null : new OperationDispatcher(port, this.seiModel.getWSBinding(), this.seiModel));
            }
        }
        return;
    }
    
    public SEIModel getModel() {
        return this.seiModel;
    }
    
    private void initStubHandlers() {
        this.stubHandlers = new HashMap();
        Map<ActionBasedOperationSignature, JavaMethodImpl> syncs = new HashMap();
        for (JavaMethodImpl m : this.seiModel.getJavaMethods()) {
            if (!m.getMEP().isAsync) {
                StubHandler handler = new StubHandler(m, this.packetFactory);
                syncs.put(m.getOperationSignature(), m);
                this.stubHandlers.put(m.getMethod(), handler);
            }
            else {
                continue;
            }
        }
        for (JavaMethodImpl jm : this.seiModel.getJavaMethods()) {
            JavaMethodImpl sync = (JavaMethodImpl)syncs.get(jm.getOperationSignature());
            if (jm.getMEP() == MEP.ASYNC_CALLBACK || jm.getMEP() == MEP.ASYNC_POLL) {
                Method i = jm.getMethod();
                StubAsyncHandler handler2 = new StubAsyncHandler(jm, sync, this.packetFactory);
                this.stubHandlers.put(i, handler2);
            }
            else {
                continue;
            }
        }
    }
    
    JavaMethodImpl resolveJavaMethod(Packet req) throws DispatchException {
        WSDLOperationMapping m = req.getWSDLOperationMapping();
        if (m == null) {
            synchronized (this) {
                m = ((this.operationDispatcher != null) ? this.operationDispatcher.getWSDLOperationMapping(req) : this.operationDispatcherNoWsdl.getWSDLOperationMapping(req));
            }
        }
        return (JavaMethodImpl)m.getJavaMethod();
    }
    
    public JavaCallInfo deserializeRequest(Packet req) {
        com.sun.xml.internal.ws.api.databinding.JavaCallInfo call = new com.sun.xml.internal.ws.api.databinding.JavaCallInfo();
        try {
            JavaMethodImpl wsdlOp = this.resolveJavaMethod(req);
            TieHandler tie = (TieHandler)this.wsdlOpMap.get(wsdlOp);
            call.setMethod(tie.getMethod());
            Object[] args = tie.readRequest(req.getMessage());
            call.setParameters(args);
        }
        catch (DispatchException e) {
            call.setException(e);
        }
        return call;
    }
    
    public JavaCallInfo deserializeResponse(Packet res, JavaCallInfo call) {
        StubHandler stubHandler = (StubHandler)this.stubHandlers.get(call.getMethod());
        try {
            return stubHandler.readResponse(res, call);
        }
        catch (Throwable e) {
            call.setException(e);
            return call;
        }
    }
    
    public WebServiceFeature[] getFeatures() {
        return null;
    }
    
    public Packet serializeRequest(JavaCallInfo call) {
        StubHandler stubHandler = (StubHandler)this.stubHandlers.get(call.getMethod());
        Packet p = stubHandler.createRequestPacket(call);
        p.setState(Packet.State.ClientRequest);
        return p;
    }
    
    public Packet serializeResponse(JavaCallInfo call) {
        Method method = call.getMethod();
        Message message = null;
        if (method != null) {
            TieHandler th = (TieHandler)this.tieHandlers.get(method);
            if (th != null) {
                return th.serializeResponse(call);
            }
        }
        if ((call.getException() instanceof DispatchException)) {
            message = ((DispatchException)call.getException()).fault;
        }
        Packet p = (Packet)this.packetFactory.createContext(message);
        p.setState(Packet.State.ServerResponse);
        return p;
    }
    
    public ClientCallBridge getClientBridge(Method method) {
        return (ClientCallBridge)this.stubHandlers.get(method);
    }
    
    public void generateWSDL(WSDLGenInfo info) {
        WSDLGenerator wsdlGen = new WSDLGenerator(this.seiModel, info.getWsdlResolver(), this.seiModel.getWSBinding(), info.getContainer(), this.seiModel.getEndpointClass(), info.isInlineSchemas(), info.isSecureXmlProcessingDisabled(), info.getExtensions());
        wsdlGen.doGeneration();
    }
    
    public EndpointCallBridge getEndpointBridge(Packet req) throws DispatchException {
        JavaMethodImpl wsdlOp = this.resolveJavaMethod(req);
        return (EndpointCallBridge)this.wsdlOpMap.get(wsdlOp);
    }
    
    Codec getCodec() {
        if (this.codec == null) {
            this.codec = ((BindingImpl)this.seiModel.getWSBinding()).createCodec();
        }
        return this.codec;
    }
    
    public ContentType encode(Packet packet, OutputStream out) throws IOException {
        return this.getCodec().encode(packet, out);
    }
    
    public void decode(InputStream in, String ct, Packet p) throws IOException {
        this.getCodec().decode(in, ct, p);
    }
    
    public JavaCallInfo createJavaCallInfo(Method method, Object[] args) {
        return new com.sun.xml.internal.ws.api.databinding.JavaCallInfo(method, args);
    }
    
    public JavaCallInfo deserializeResponse(MessageContext message, JavaCallInfo call) {
        return this.deserializeResponse((Packet)message, call);
    }
    
    public JavaCallInfo deserializeRequest(MessageContext message) {
        return this.deserializeRequest((Packet)message);
    }
    
    public MessageContextFactory getMessageContextFactory() {
        return this.packetFactory;
    }
}
